from typing import Annotated, Literal

from typing_extensions import TypeAliasType

from pydantic import BaseModel, Field, model_validator

from olmoearth_run.shared.split_proportions import validate_split_proportions


class RandomDataSplitter(BaseModel):
    """Data splitter that assigns splits based on random sampling."""
    name: Literal["random"]
    train_prop: float = Field(ge=0, le=1, description="The proportion of data to assign to the training split.")
    val_prop: float = Field(ge=0, le=1, description="The proportion of data to assign to the validation split.")
    test_prop: float = Field(ge=0, le=1, description="The proportion of data to assign to the test split.")
    seed: int | None = Field(default=42, description="The seed for the random number generator.")

    @model_validator(mode='after')
    def validate_proportions_sum_to_one(self) -> 'RandomDataSplitter':
        """Ensure that train_prop + val_prop + test_prop == 1"""
        validate_split_proportions(self.train_prop, self.val_prop, self.test_prop)
        return self


class SpatialDataSplitter(BaseModel):
    """Data splitter that assigns splits based on spatial grid cell location."""
    name: Literal["spatial"]
    grid_size: int = Field(ge=1, description="The size of the grid cells in pixels.")
    train_prop: float = Field(ge=0, le=1, description="The proportion of data to assign to the training split.")
    val_prop: float = Field(ge=0, le=1, description="The proportion of data to assign to the validation split.")
    test_prop: float = Field(ge=0, le=1, description="The proportion of data to assign to the test split.")

    @model_validator(mode='after')
    def validate_proportions_sum_to_one(self) -> 'SpatialDataSplitter':
        """Ensure that train_prop + val_prop + test_prop == 1."""
        validate_split_proportions(self.train_prop, self.val_prop, self.test_prop)
        return self


DataSplitter = TypeAliasType(
    "DataSplitter",
    Annotated[
        RandomDataSplitter | SpatialDataSplitter,
        Field(discriminator="name")
    ]
)



from pydantic import BaseModel, Field

from olmoearth_run.shared.models.config.input_preprocessing.samplers import TrainingSampler
from olmoearth_run.shared.models.config.input_preprocessing.transforms import Transform


class DefaultInputPreprocessing(BaseModel):
    input_size: int = Field(ge=1, description="Extracts square crops from the provided images at this size, which become individual inputs in inference batches after the transform pipeline is complete.")
    transforms: list[Transform] = Field(default_factory=lambda: [], description="The transforms to apply to the input tensors before they are provided to the model.")


class TrainingSplitInputPreprocessing(BaseModel):
    input_size: int | None = Field(default=None, ge=1, description="Overrides the `input_size` from the default config for this split.")
    transforms: list[Transform] = Field(default_factory=lambda: [], description="The transforms to apply to the input tensors before they are provided to the model. Replaces the default transforms; takes precendence over `additional_transforms`.")
    additional_transforms: list[Transform] = Field(default_factory=lambda: [], description="Additional transforms to apply to the input tensors (beyond the default transforms) before they are provided to the model.")
    sampler: TrainingSampler | None = Field(default=None, description="When provided, can be used to sample down the provided data to a smaller set based on the given strategy.")


class PredictInputPreprocessing(BaseModel):
    input_size: int | None = Field(default=None, ge=1, description="Overrides the `input_size` from the default config for this split.")
    overlap_pixels: int = Field(ge=0, description="Helps define a stride with a given amount of overlap between inputs cropped out of source tensors. Is also used later when merging inference products into a single output.")
    transforms: list[Transform] = Field(default_factory=lambda: [], description="The transforms to apply to the input tensors before they are provided to the model. Replaces the default transforms; takes precendence over `additional_transforms`.")
    additional_transforms: list[Transform] = Field(default_factory=lambda: [], description="Additional transforms to apply to the input tensors (beyond the default transforms) before they are provided to the model.")


class InputPreprocessing(BaseModel):
    """
    Defines how input tensors are prepared before being provided to the model.
    Each training dataset split (train, val, test) can have its own preprocessing configuration,
    as can input data at inference ("predict") time.
    All take their base configuration from "default", which can be overridden for each group.
    """
    default: DefaultInputPreprocessing = Field(description="Default input preprocessing configuration")
    train: TrainingSplitInputPreprocessing | None = None
    val: TrainingSplitInputPreprocessing | None = None
    test: TrainingSplitInputPreprocessing | None = None
    predict: PredictInputPreprocessing | None = None



from typing import Annotated, Literal

from typing_extensions import TypeAliasType

from pydantic import BaseModel, Field


class UseUTMProjection(BaseModel):
    method: Literal["use_utm"]


class CRSProjection(BaseModel):
    method: Literal["crs"]
    crs: str = Field(description="The CRS code of the output projection.")
    x_resolution: float = Field(gt=0, description="The x resolution of the output projection, in projection units.")
    y_resolution: float = Field(lt=0, description="The y resolution of the output projection, in projection units.")


Projection = TypeAliasType(
    "Projection",
    Annotated[
        UseUTMProjection | CRSProjection,
        Field(discriminator="method")
    ]
)



from rslearn.config import dataset

from olmoearth_run.runner.models.operational_context import OperationalContext
from olmoearth_run.shared.models.config.olmoearth_config import OlmoEarthConfig
from olmoearth_run.runner.tools.config_transpilers.modality_layers import ModalityLayers
from olmoearth_run.runner.tools.config_transpilers.label_output_layers import LabelOutputLayers


def to_dataset_config(olmoearth_config: OlmoEarthConfig, ops_context: OperationalContext) -> dataset.DatasetConfig:
    return dataset.DatasetConfig(
        layers={
            **ModalityLayers.generate_modality_layers(olmoearth_config, ops_context),
            **LabelOutputLayers.generate_label_and_output_layers(olmoearth_config, ops_context)
        },
    )



from typing import Annotated, Literal

from typing_extensions import TypeAliasType

from pydantic import BaseModel, Field


class NoopSampler(BaseModel):
    """No-op sampler that returns the full dataset."""
    name: Literal["noop"]


class ChooseNSampler(BaseModel):
    """Sampler that chooses a random subset of annotation tasks."""
    name: Literal["choose_n"]
    n: int = Field(ge=0, description="The number of samples to take")


AnnotationSampler = TypeAliasType(
    "AnnotationSampler",
    Annotated[
        NoopSampler | ChooseNSampler,
        Field(discriminator="name")
    ]
)



from typing import Annotated, Literal

from typing_extensions import TypeAliasType

from pydantic import BaseModel, Field


class AdamW(BaseModel):
    name: Literal["adamw"]
    lr: float = Field(ge=0, description="The learning rate to use for the optimizer.")


Optimizer = TypeAliasType(
    "Optimizer",
    Annotated[
        AdamW,
        Field(discriminator="name")
    ]
)



from enum import StrEnum


class ModalityName(StrEnum):
    SENTINEL2_L2A = "sentinel2_l2a"


SENTINEL2_L2A_BANDS = ["B01", "B02", "B03", "B04", "B05", "B06", "B07", "B08", "B8A", "B09", "B11", "B12"]

LABEL_LAYER_NAME = "label"
OUTPUT_LAYER_NAME = "output"

STAC_EO_CLOUD_COVER = "eo:cloud_cover"



from pydantic import BaseModel, Field
import yaml # type: ignore[import-untyped]

from olmoearth_run.shared.models.config.data.data import Data
from olmoearth_run.shared.models.config.input_preprocessing.input_preprocessing import InputPreprocessing
from olmoearth_run.shared.models.config.labeled_data_prep.labeled_data_prep import LabeledDataPrep
from olmoearth_run.shared.models.config.model.model import Model
from olmoearth_run.shared.models.config.prediction_requests.prediction_requests import PredictionRequests
from olmoearth_run.shared.models.config.training.training import Training


class OlmoEarthConfig(BaseModel):
    """
    Unified configuration for OlmoEarth: data requirements, model, dataset prep, training, prediction.
    """

    config_version: str = Field(description="The version of the configuration")
    data: Data = Field(description="Specifies the required data modalities, temporality of the data to retrieve, and the output schema (when a `model` is defined).")
    model: Model | None = Field(default=None, description="Defines the model architecture and the tasks to perform during inference.")
    input_preprocessing: InputPreprocessing | None = Field(default=None, description="Defines how input tensors are prepared or transformed before being provided to the model.")
    labeled_data_prep: LabeledDataPrep | None = Field(default=None, description="Controls the preparation of labeled data for training.")
    training: Training | None = Field(default=None, description="Configuration for the training")
    prediction_requests: PredictionRequests | None = Field(default=None, description="Controls the prediction requests for the model.")

    @classmethod
    def from_yaml(cls, yaml_content: str) -> "OlmoEarthConfig":
        """
        Parse OlmoEarth configuration from a YAML string.

        Raises:
            ValueError: If the YAML content is malformed or doesn't match the expected structure
        """
        try:
            raw_config = yaml.safe_load(yaml_content)
        except yaml.YAMLError as e:
            raise ValueError("Invalid YAML in olmoearth_run_config") from e

        try:
            return cls.model_validate(raw_config)
        except Exception as e:  # Catch ValidationError and other pydantic errors
            raise ValueError(f"Invalid olmoearth_config structure. {e}") from e



from enum import StrEnum

from pydantic import BaseModel, Field


class DataSource(StrEnum):
    PLANETARY_COMPUTER = "planetary_computer"


class OperationalContext(BaseModel):
    """Environment/context-specific information necessary for controlling behavior of dataset and model operations."""

    data_source: DataSource = Field(default=DataSource.PLANETARY_COMPUTER, description="The data source for acquiring imagery")



from typing import Annotated, Literal

from typing_extensions import TypeAliasType

from pydantic import BaseModel, Field

from olmoearth_run.shared.models.config.model.decoders import SegmentationDecoder


class SegmentationTask(BaseModel):
    name: Literal["segmentation"]
    decoder: SegmentationDecoder


# TODO: classification, per-pixel regression, regression, and detection tasks


ModelTask = TypeAliasType(
    "ModelTask",
    Annotated[
        SegmentationTask,
        Field(discriminator="name")
    ]
)



from typing import Annotated, Literal

from typing_extensions import TypeAliasType

from pydantic import BaseModel, Field


class NoopPostprocessor(BaseModel):
    """Postprocessor that does nothing."""
    name: Literal["noop"]


class CombineGeotiff(BaseModel):
    """Postprocessor that combines the geotiffs into a single geotiff."""
    name: Literal["combine_geotiff"]


Postprocessor = TypeAliasType(
    "Postprocessor",
    Annotated[
        NoopPostprocessor | CombineGeotiff,
        Field(discriminator="name")
    ]
)


class Postprocessors(BaseModel):
    window: Postprocessor = Field(description="Controls the postprocessing of the prediction results for a single window.")
    partition: Postprocessor = Field(description="Controls the postprocessing of all the window results for a single partition.")
    request: Postprocessor = Field(description="Controls the postprocessing of all the partition results for the entire request.")



"""
Used (optionally) to winnow down items from training data splits at training time.
"""

from typing import Annotated, Literal

from typing_extensions import TypeAliasType

from pydantic import BaseModel, Field


class RandomSampler(BaseModel):
    name: Literal["random"]
    num_samples: int = Field(ge=0, description="The number of samples to take")
    replace: bool = Field(default=False, description="Whether sampled items can be sampled again")


TrainingSampler = TypeAliasType(
    "TrainingSampler",
    Annotated[
        RandomSampler,
        Field(discriminator="name")
    ]
)
